package logic

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/uuid"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type RetrievePlatformUserLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext

	logx.Logger
}

func NewRetrievePlatformUserLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RetrievePlatformUserLogic {
	return &RetrievePlatformUserLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// RetrievePlatformUser 查询或创建用户（此时不查询平台用户信息）
func (l *RetrievePlatformUserLogic) RetrievePlatformUser(in *pb.PlatformUserReq) (*pb.PlatformUserResp, error) {
	var username string
	var dbPlatformUser *model.UserPlatform

	err := l.svcCtx.UserPlatformModel.Transaction(l.ctx, nil, func(tx *gorm.DB) error {
		var err error
		if dbPlatformUser, err = l.svcCtx.UserPlatformModel.FindOneByPlatformAndPUid(l.ctx, tx, in.Platform, in.PUid); err != nil {
			if !errors.Is(err, model.ErrNotFound) {
				return err
			}
		}
		if dbPlatformUser != nil {
			if one, err := l.svcCtx.UserModel.FindOneTx(l.ctx, tx, dbPlatformUser.UserId); err != nil {
				username = one.Username
			}
			return nil
		}
		// insert
		newId := uuid.NextId()
		if err := l.svcCtx.UserModel.InsertTx(l.ctx, tx, &model.User{Id: newId}); err != nil {
			return errors.Wrap(err, "插入用户数据失败")
		}
		dbPlatformUser = &model.UserPlatform{
			Id:       uuid.NextId(),
			UserId:   newId,
			Platform: in.Platform,
			PUid:     in.PUid,
			PInfo:    "{}",
		}
		if err := l.svcCtx.UserPlatformModel.InsertTx(l.ctx, tx, dbPlatformUser); err != nil {
			return errors.Wrap(err, "插入平台用户数据失败")
		}
		return nil
	})
	if err != nil {
		return nil, errors.Wrapf(err, "查询或创建用户-事务执行失败, err:%+v", err)
	}
	return &pb.PlatformUserResp{
		User: l.buildPBUser(username, dbPlatformUser),
	}, nil
}

func (l *RetrievePlatformUserLogic) buildPBUser(username string, dbPlatformUser *model.UserPlatform) *pb.User {
	return &pb.User{
		Id:       dbPlatformUser.UserId,
		Username: username,
		Platform: dbPlatformUser.Platform,
		PUid:     dbPlatformUser.PUid,
		PUname:   dbPlatformUser.PUname,
		PAvatar:  dbPlatformUser.PAvatar,
		PInfo:    dbPlatformUser.PInfo,
	}
}
